Type: Package
Package: crosstalk
Title: Inter-Widget Interactivity for HTML Widgets
Version: 1.2.2
Authors@R: c(
    person("Joe", "Cheng", , "joe@posit.co", role = "aut"),
    person("Carson", "Sievert", , "carson@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-4958-2844")),
    person("Posit Software, PBC", role = c("cph", "fnd")),
    person(, "jQuery Foundation", role = "cph",
           comment = "jQuery library and jQuery UI library"),
    person(, "jQuery contributors", role = c("ctb", "cph"),
           comment = "jQuery library; authors listed in inst/www/shared/jquery-AUTHORS.txt"),
    person("Mark", "Otto", role = "ctb",
           comment = "Bootstrap library"),
    person("Jacob", "Thornton", role = "ctb",
           comment = "Bootstrap library"),
    person(, "Bootstrap contributors", role = "ctb",
           comment = "Bootstrap library"),
    person(, "Twitter, Inc", role = "cph",
           comment = "Bootstrap library"),
    person("Brian", "Reavis", role = c("ctb", "cph"),
           comment = "selectize.js library"),
    person("Kristopher Michael", "Kowal", role = c("ctb", "cph"),
           comment = "es5-shim library"),
    person(, "es5-shim contributors", role = c("ctb", "cph"),
           comment = "es5-shim library"),
    person("Denis", "Ineshin", role = c("ctb", "cph"),
           comment = "ion.rangeSlider library"),
    person("Sami", "Samhuri", role = c("ctb", "cph"),
           comment = "Javascript strftime library")
  )
Description: Provides building blocks for allowing HTML widgets to
    communicate with each other, with Shiny or without (i.e. static .html
    files). Currently supports linked brushing and filtering.
License: MIT + file LICENSE
URL: https://rstudio.github.io/crosstalk/,
        https://github.com/rstudio/crosstalk
BugReports: https://github.com/rstudio/crosstalk/issues
Imports: htmltools (>= 0.3.6), jsonlite, lazyeval, R6
Suggests: bslib, ggplot2, sass, shiny, testthat (>= 2.1.0)
Config/Needs/website: jcheng5/d3scatter, DT, leaflet, rmarkdown
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-08-26 22:06:47 UTC; cpsievert
Author: Joe Cheng [aut],
  Carson Sievert [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4958-2844>),
  Posit Software, PBC [cph, fnd],
  jQuery Foundation [cph] (jQuery library and jQuery UI library),
  jQuery contributors [ctb, cph] (jQuery library; authors listed in
    inst/www/shared/jquery-AUTHORS.txt),
  Mark Otto [ctb] (Bootstrap library),
  Jacob Thornton [ctb] (Bootstrap library),
  Bootstrap contributors [ctb] (Bootstrap library),
  Twitter, Inc [cph] (Bootstrap library),
  Brian Reavis [ctb, cph] (selectize.js library),
  Kristopher Michael Kowal [ctb, cph] (es5-shim library),
  es5-shim contributors [ctb, cph] (es5-shim library),
  Denis Ineshin [ctb, cph] (ion.rangeSlider library),
  Sami Samhuri [ctb, cph] (Javascript strftime library)
Maintainer: Carson Sievert <carson@posit.co>
Repository: CRAN
Date/Publication: 2025-08-26 23:50:02 UTC
Built: R 4.6.0; ; 2026-04-27 13:32:17 UTC; unix

# Generated by roxygen2: do not edit by hand

export(ClientValue)
export(SharedData)
export(animation_options)
export(bscols)
export(crosstalkLibs)
export(filter_checkbox)
export(filter_select)
export(filter_slider)
export(getDefaultReactiveDomain)
export(is.SharedData)
export(maintain_selection)
export(scale_color_selection)
export(scale_fill_selection)
export(selection_factor)
import(R6)
import(htmltools)
importFrom(stats,na.omit)
importFrom(stats,setNames)
importFrom(utils,packageVersion)

# Crosstalk

<!-- badges: start -->
[![CRAN](https://www.r-pkg.org/badges/version/crosstalk)](https://CRAN.R-project.org/package=crosstalk)
[![R-CMD-check](https://github.com/rstudio/crosstalk/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/rstudio/crosstalk/actions)
<!-- badges: end -->

Crosstalk is a package for R that enhances the [htmlwidgets](http://www.htmlwidgets.org) package. It extends htmlwidgets with a set of classes, functions, and conventions for implementing cross-widget interactions (currently, linked brushing and filtering).

Find out more at the documentation website: http://rstudio.github.io/crosstalk/

## Building JavaScript assets

(This section is only for developers who intend to modify the JavaScript source code in Crosstalk itself.)

The JavaScript source code in this package lives under `javascript/`, however the copy that is actually loaded and used during runtime is in minified form at `inst/www/js/`. Anytime you make changes to `javascript/` source files, you must rebuild the minified JS.

To set up your repo for building the minified JS:

1. First install [nvm](https://github.com/nvm-sh/nvm) if you don't have it already.
2. In the crosstalk repo's root directory, run `nvm install`; this will install the version of Node.js we need.
3. Run `nvm use` to switch to our version of Node.js.
4. Run `npm install` to install all Node.js dependencies.

To actually build the minified JS:

1. Run `nvm use` (only needed once per terminal session).
2. Run `node node_modules/.bin/grunt` (or if you have installed `grunt-cli` globally, you can just run `grunt`).

This will run unit tests, lint, and build the JavaScript dist bundle. When making changes to the JavaScript code, you must always do this first, and then build the R package as normal.

shinyInstalled <- local({
  detected <- FALSE
  function() {
    if (getOption("crosstalk.shiny.suppressed", FALSE)) {
      # If crosstalk.shiny.suppressed is TRUE, we're in a special testing
      # mode and want to act as if shiny isn't installed
      return(FALSE)
    }

    if (detected) {
      return(TRUE)
    }

    # Test that Shiny is installed AND shiny::reactive hasn't been NULLed out
    if (nzchar(system.file(package = "shiny"))) {
      detected <<- TRUE
      return(TRUE)
    }

    return(FALSE)
  }
})

stopIfNotShiny <- function(message) {
  # So that we catch incorrect calls to `stopIfNotShiny` even
  # if Shiny is installed
  stopifnot(!missing(message))

  if (!shinyInstalled()) {
    stop(call. = FALSE, message)
  }
}

#' Get default reactive domain
#'
#' Pass-through to [shiny::getDefaultReactiveDomain()], unless the shiny package
#' is not installed, in which case `NULL` is returned.
#'
#' @keywords internal
#' @export
getDefaultReactiveDomain <- function() {
  if (shinyInstalled()) {
    shiny::getDefaultReactiveDomain()
  } else {
    NULL
  }
}

is.reactive <- function(x) {
  if (shinyInstalled()) {
    shiny::is.reactive(x)
  } else {
    FALSE
  }
}

reactiveValues <- function(...) {
  if (shinyInstalled()) {
    shiny::reactiveValues(...)
  } else {
    list(...)
  }
}

#' @import htmltools
init <- function() {
  htmltools::attachDependencies(
    list(),
    crosstalkLibs()
  )
}

#' Crosstalk dependencies
#'
#' List of \code{\link[htmltools]{htmlDependency}} objects necessary for
#' Crosstalk to function. Intended for widget authors.
#' @importFrom stats na.omit setNames
#' @importFrom utils packageVersion
#' @export
crosstalkLibs <- function() {
  list(
    jqueryLib(),
    htmltools::htmlDependency(
      name = "crosstalk",
      version = packageVersion("crosstalk"),
      package = "crosstalk",
      src = "www",
      script = "js/crosstalk.min.js",
      stylesheet = "css/crosstalk.css"
    )
  )
}

#' ClientValue object
#'
#' @description
#' An object that can be used in a \href{https://shiny.rstudio.com/}{Shiny} server
#' function to get or set a crosstalk variable that exists on the client. The
#' client copy of the variable is the canonical copy, so there is no direct
#' "set" method that immediately changes the value; instead, there is a
#' `sendUpdate` method that sends a request to the browser to change the value,
#' which will then cause the new value to be relayed back to the server.
#'
#' This object is used to implement \code{\link{SharedData}} and should not need
#' to be used directly by users.
#'
#' @examples
#' library(shiny)
#'
#' server <- function(input, output, session) {
#'   cv <- ClientValue$new("var1", "group1")
#'
#'   r <- reactive({
#'     # Don't proceed unless cv$get() is a non-NULL value
#'     validate(need(cv$get(), message = FALSE))
#'
#'     runif(cv$get())
#'   })
#'
#'   observeEvent(input$click, {
#'     cv$sendUpdate(NULL)
#'   })
#' }
#'
#' @import R6
#' @export
ClientValue <- R6Class(
  "ClientValue",
  private = list(
    .session = "ANY",
    .name = "ANY",
    .group = "ANY",
    .qualifiedName = "ANY",
    .rv = "ANY"
  ),
  public = list(
    #' @description
    #' Creates a new ClientValue object to reflect the crosstalk variable
    #' specified by `group` and `name`.
    #'
    #' @param name The name of the crosstalk variable.
    #' @param group The name of the crosstalk variable group.
    #' @param session The Shiny session to connect to; defaults to the current
    #'   session.
    initialize = function(name, group = "default", session = shiny::getDefaultReactiveDomain()) {
      if (!missing(session) || shinyInstalled()) {
        private$.session <- session
      } else {
        # If session wasn't explicitly provided and Shiny isn't installed, we can't use
        # the default value for session because it uses a shiny:: function. Instead,
        # set it to NULL. This is hacky and weird but ClientValue doesn't really need
        # to work well if Shiny isn't installed, it just needs to not throw.
        private$.session <- NULL
      }
      private$.name <- name
      private$.group <- group
      private$.qualifiedName <- paste0(".clientValue-", group, "-", name)
    },

    #' @description
    #' Read the value. This is a reactive operation akin to reading a reactive
    #' value, and so can only be done in a reactive context (e.g. in a
    #' `shiny::reactive()`, `shiny::observe()`, or `shiny::isolate()` block).
    get = function() {
      private$.session$input[[private$.qualifiedName]]
    },

    #' @description
    #' Send a message to the browser asking it to update the crosstalk var to
    #' the given value. This update does not happen synchronously, that is, a
    #' call to `get()` immediately following `sendUpdate(value)` will not
    #' reflect the new value.
    #' @param value The new value for the crosstalk variable. Must be
    #'   serializable as JSON using `jsonlite`.
    sendUpdate = function(value) {
      stopIfNotShiny("ClientValue$sendUpdate() requires the shiny package")
      private$.session$sendCustomMessage("update-client-value", list(
        name = private$.name,
        group = private$.group,
        value = value
      ))
    }
  )
)


createUniqueId <- function (bytes, prefix = "", suffix = "") {
  paste(prefix, paste(format(as.hexmode(sample(256, bytes,
    replace = TRUE) - 1), width = 2), collapse = ""),
    suffix, sep = "")
}


#' Shared data frame
#'
#' @description
#' An R6 class that represents a shared data frame, or sufficiently data
#' frame-like object.
#'
#' The primary use for \code{SharedData} is to be passed to Crosstalk-compatible
#' widgets in place of a data frame. Each \code{SharedData$new(...)} call makes
#' a new "group" of widgets that link to each other, but not to widgets in other
#' groups. You can also use a \code{SharedData} object from Shiny code in order
#' to react to filtering and brushing from non-widget visualizations (like
#' ggplot2 plots).
#'
#' @import R6
#' @export
SharedData <- R6Class(
  "SharedData",
  private = list(
    .data = "ANY",
    .key = "ANY",
    .filterCV = "ANY",
    .selectionCV = "ANY",
    .rv = "ANY",
    .group = "ANY",
    # Update selection without sending event
    .updateSelection = function(value) {
      force(value)
      `$<-`(private$.rv, "selected", value)
    }
  ),
  public = list(
    #' @param data A data frame-like object, or a Shiny
    #'   \link[shiny:reactive]{reactive expression} that returns a data
    #'   frame-like object.
    #' @param key Character vector or one-sided formula that indicates the name
    #'   of the column that represents the key or ID of the data frame. These
    #'   \emph{must} be unique, and ideally will be something intrinsic to the
    #'   data (a proper ID) rather than a transient property like row index.
    #'
    #'   If \code{NULL}, then \code{row.names(data)} will be used.
    #' @param group The "identity" of the Crosstalk group that widgets will join
    #'   when you pass them this \code{SharedData} object. In some cases, you
    #'   will want to have multiple independent \code{SharedData} objects link
    #'   up to form a single web of widgets that all share selection and
    #'   filtering state; in those cases, you'll give those \code{SharedData}
    #'   objects the same group name. (One example: in Shiny, ui.R and server.R
    #'   might each need their own \code{SharedData} instance, even though
    #'   they're intended to represent a single group.)
    initialize = function(data, key = NULL, group = createUniqueId(4, prefix = "SharedData")) {
      private$.data <- data
      private$.filterCV <- ClientValue$new("filter", group)
      private$.selectionCV <- ClientValue$new("selection", group)
      private$.rv <- reactiveValues()
      private$.group <- group

      if (inherits(key, "formula")) {
        private$.key <- key
      } else if (is.character(key)) {
        private$.key <- key
      } else if (is.function(key)) {
        private$.key <- key
      } else if (is.null(key)) {
        private$.key <- key
      } else {
        stop("Unknown key type")
      }

      if (is.reactive(private$.data)) {
        shiny::observeEvent(private$.data(), {
          self$clearSelection()
        })
      }

      domain <- getDefaultReactiveDomain()
      if (!is.null(domain)) {
        observe({
          selection <- private$.selectionCV$get()
          if (!is.null(selection) && length(selection) > 0) {
            private$.updateSelection(self$key() %in% selection)
          } else {
            private$.updateSelection(NULL)
          }
        })
      }
    },
    #' @description Return the data frame that was used to create this
    #' \code{SharedData} instance. If a reactive expression, evaluate the
    #' reactive expression. Equivalent to \code{SharedData$data(FALSE, FALSE,
    #' FALSE)}.
    origData = function() {
      if (is.reactive(private$.data)) {
        private$.data()
      } else {
        private$.data
      }
    },
    #' @description Returns the value of \code{group} that was used to create
    #' this instance.
    groupName = function() {
      private$.group
    },
    #' @description Returns the vector of key values. Filtering is not applied.
    key = function() {
      df <- if (is.reactive(private$.data)) {
        private$.data()
      } else {
        private$.data
      }

      key <- private$.key
      if (inherits(key, "formula"))
        lazyeval::f_eval(key, df)
      else if (is.character(key))
        key
      else if (is.function(key))
        key(df)
      else if (!is.null(row.names(df)))
        row.names(df)
      else if (nrow(df) > 0)
        as.character(1:nrow(df))
      else
        character()
    },
    #' @description
    #' Return the data (or read and return the data if the data is a Shiny
    #' reactive expression).
    #'
    #' When running in Shiny, calling \code{data()} is a reactive operation that
    #' will invalidate if the selection or filter change (assuming that
    #' information was requested), or if the original data is a reactive
    #' expression that has invalidated.
    #' @param withSelection If `TRUE`, add a \code{selection_} column with
    #'   logical values indicating which rows are in the current selection, or
    #'   \code{NA} if no selection is currently active.
    #' @param withFilter If `TRUE` (the default), only return rows that are part
    #'   of the current filter settings, if any.
    #' @param withKey If `TRUE`, add a \code{key_} column with the key values of
    #'   each row (normally not needed since the key is either one of the other
    #'   columns or else just the row names).
    data = function(withSelection = FALSE, withFilter = TRUE, withKey = FALSE) {
      df <- if (is.reactive(private$.data)) {
        private$.data()
      } else {
        private$.data
      }

      op <- options(shiny.suppressMissingContextError = TRUE)
      on.exit(options(op), add = TRUE)

      if (withSelection) {
        if (is.null(private$.rv$selected) || length(private$.rv$selected) == 0) {
          df$selected_ = NA
        } else {
          # TODO: Warn if the length of _selected is different?
          df$selected_ <- private$.rv$selected
        }
      }

      if (withKey) {
        df$key_ <- self$key()
      }

      if (withFilter) {
        if (!is.null(private$.filterCV$get())) {
          df <- df[self$key() %in% private$.filterCV$get(),]
        }
      }

      df
    },
    # Public API for selection getting/setting. Setting a selection will
    # cause an event to be propagated to the client.
    #' @description Get or set the current selection in the client.
    #'
    #' If called without arguments, returns a logical vector of rows that are
    #' currently selected (brushed), or \code{NULL} if no selection exists.
    #' Intended to be called from a Shiny reactive context, and invalidates
    #' whenever the selection changes.
    #'
    #' If called with one or two arguments, sets the selection based on the
    #' given value indirectly, by sending the value to the web browser (assumes
    #' an active Shiny app or Shiny R Markdown document).
    #'
    #' @param value If provided, a logical vector of `nrow(origData())` length,
    #'   indicating which rows are currently selected (brushed).
    #'
    #' @param ownerId Set this argument to the `outputId` of a widget if
    #'   conceptually that widget "initiated" the selection (prevents that
    #'   widget from clearing its visual selection box, which is normally
    #'   cleared when the selection changes). For example, if setting the
    #'   selection based on a [shiny::plotOutput()] brush, then
    #'   `ownerId` should be the `outputId` of that `plotOutput`.
    selection = function(value, ownerId = "") {
      stopIfNotShiny("SharedData$selection() requires the shiny package")

      if (missing(value)) {
        return(private$.rv$selected)
      } else {
        # TODO: Should we even update the server at this time? Or do we
        # force all such events to originate in the client (much like
        # updateXXXInput)?

        # .updateSelection needs logical array of length nrow(data)
        # .selectionCV$sendUpdate needs character array of keys
        shiny::isolate({
          if (is.null(value)) {
            private$.updateSelection(NULL)
            private$.selectionCV$sendUpdate(NULL)
          } else {
            key <- self$key()
            if (is.character(value)) {
              private$.updateSelection(key %in% value)
              private$.selectionCV$sendUpdate(value)
            } else if (is.logical(value)) {
              private$.updateSelection(value)
              private$.selectionCV$sendUpdate(key[value])
            } else if (is.numeric(value)) {
              self$selection(1:nrow(self$data(FALSE)) %in% value)
            }
          }
        })
      }
    },
    #' @description Clears the selection indirectly, by sending an instruction
    #' to the client that it should do so.
    #'
    #' @param ownerId See the [SharedData$selection()] method.
    clearSelection = function(ownerId = "") {
      stopIfNotShiny("SharedData$clearSelection() requires the shiny package")
      self$selection(list(), ownerId = "")
    }
  )
)

#' Check if an object is \code{SharedData}
#'
#' Check if an object is an instance of \code{\link{SharedData}} or not.
#'
#' @param x The object that may or may not be an instance of \code{SharedData}
#' @return logical
#'
#' @export
is.SharedData <- function(x) {
  inherits(x, "SharedData")
}

set.seed(64327883)

run_main_test <- function() {
  expect_identical(
    SharedData$new(mtcars)$key(),
    row.names(mtcars)
  )
  expect_identical(
    SharedData$new(mtcars, NULL)$key(),
    row.names(mtcars)
  )

  expect_identical(SharedData$new(mtcars)$data(), mtcars)
  expect_identical(SharedData$new(mtcars)$origData(), mtcars)

  expect_identical(
    SharedData$new(mtcars)$data(TRUE, FALSE, TRUE),
    cbind(mtcars, selected_ = NA, key_ = row.names(mtcars),
      stringsAsFactors = FALSE)
  )

  mtcarsWithNames <- mtcars
  row.names(mtcarsWithNames) <- NULL
  mtcarsWithNames <- cbind(id = row.names(mtcars), mtcarsWithNames,
    stringsAsFactors = FALSE)

  expect_identical(
    SharedData$new(mtcarsWithNames, mtcarsWithNames[["id"]])$key(),
    row.names(mtcars)
  )

  expect_identical(
    SharedData$new(iris, group = "Iris")$groupName(),
    "Iris"
  )
}

test_that("SharedData basic scenarios", {
  skip_if_not_installed("shiny")
  run_main_test()

  expect_error(SharedData$new(iris)$selection(), "reactive")
  shiny::isolate(expect_identical(SharedData$new(iris)$selection(), NULL))
})

test_that("SharedData basic scenarios with shiny 'uninstalled'", {
  skip_if_not_installed("shiny")

  # Force shinyIsInstalled() to return false
  op <- options(crosstalk.shiny.suppressed = TRUE)
  on.exit(options(op), add = TRUE)

  # After this test is over, unload shiny to undo our breakage (below)
  on.exit(unloadNamespace("shiny"), add = TRUE)

  # Break Shiny, so if Shiny functions are called they won't succeed, as
  # if Shiny is not installed.
  pkgEnv <- asNamespace("shiny")
  for (nm in ls(pkgEnv)) {
    unlockBinding(nm, pkgEnv)
    pkgEnv[[nm]] <- NULL
  }

  run_main_test()
  expect_error(SharedData$new(iris)$selection(), "requires.*shiny")

  # While we're at it, run examples
  library(crosstalk)
  helpPath <- file.path(system.file("help", package = "crosstalk"), "crosstalk")
  if (file.exists(paste0(helpPath, ".rdx"))) {
    topics <- names(tools:::fetchRdDB(helpPath))
    for (topic in topics) {
      suppressWarnings({
        eval(bquote(example(.(topic), package = "crosstalk")))
      })
    }
  }
})

test_that("Shiny modules support SharedData", {
  sess <- shiny::MockShinySession$new()
  sd1 <- shiny::withReactiveDomain(sess, {
    SharedData$new(cars, group = "foo")
  })
  sd2 <- shiny::withReactiveDomain(sess$makeScope("module1"), {
    SharedData$new(cars, group = "foo")
  })

  sess$setInputs(".clientValue-foo-selection" = c("1", "2"))

  res <- c(TRUE, TRUE, rep_len(FALSE, nrow(cars) - 2))
  expect_identical(shiny::isolate(sd1$selection()), res)
  expect_identical(shiny::isolate(sd2$selection()), res)

  res2 <- rev(res)
  sess$setInputs(".clientValue-foo-selection" = as.character(which(res2)))

  expect_identical(shiny::isolate(sd1$selection()), res2)
  expect_identical(shiny::isolate(sd2$selection()), res2)

})

set.seed(64327883)

run_main_test <- function() {
  expect_identical(
    SharedData$new(mtcars)$key(),
    row.names(mtcars)
  )
  expect_identical(
    SharedData$new(mtcars, NULL)$key(),
    row.names(mtcars)
  )

  expect_identical(SharedData$new(mtcars)$data(), mtcars)
  expect_identical(SharedData$new(mtcars)$origData(), mtcars)

  expect_identical(
    SharedData$new(mtcars)$data(TRUE, FALSE, TRUE),
    cbind(mtcars, selected_ = NA, key_ = row.names(mtcars),
      stringsAsFactors = FALSE)
  )

  mtcarsWithNames <- mtcars
  row.names(mtcarsWithNames) <- NULL
  mtcarsWithNames <- cbind(id = row.names(mtcars), mtcarsWithNames,
    stringsAsFactors = FALSE)

  expect_identical(
    SharedData$new(mtcarsWithNames, mtcarsWithNames[["id"]])$key(),
    row.names(mtcars)
  )

  expect_identical(
    SharedData$new(iris, group = "Iris")$groupName(),
    "Iris"
  )
}

test_that("SharedData basic scenarios", {
  run_main_test()

  expect_error(SharedData$new(iris)$selection(), "reactive")
  shiny::isolate(expect_identical(SharedData$new(iris)$selection(), NULL))
})

test_that("SharedData basic scenarios with shiny 'uninstalled'", {

  # Force shinyIsInstalled() to return false
  op <- options(crosstalk.shiny.suppressed = TRUE)
  on.exit(options(op), add = TRUE)

  # After this test is over, unload shiny to undo our breakage (below)
  on.exit(unloadNamespace("shiny"), add = TRUE)

  # Break Shiny, so if Shiny functions are called they won't succeed, as
  # if Shiny is not installed.
  pkgEnv <- asNamespace("shiny")
  for (nm in ls(pkgEnv)) {
    unlockBinding(nm, pkgEnv)
    pkgEnv[[nm]] <- NULL
  }

  run_main_test()
  expect_error(SharedData$new(iris)$selection(), "requires.*shiny")

  # While we're at it, run examples
  library(crosstalk)
  helpPath <- file.path(system.file("help", package = "crosstalk"), "crosstalk")
  if (file.exists(paste0(helpPath, ".rdx"))) {
    topics <- names(tools:::fetchRdDB(helpPath))
    for (topic in topics) {
      suppressWarnings({
        eval(bquote(example(.(topic), package = "crosstalk")))
      })
    }
  }
})

#' ggplot2 helpers
#'
#' Add \code{scale_fill_selection()} or \code{scale_color_selection} to a ggplot
#' to customize the scale for fill or color, respectively, for linked brushing.
#' Use \code{selection_factor} to turn logical vectors representing selection,
#' to a factor with the levels ordered for use with ggplot2 bar stacking.
#'
#' @param color_false The color that should be mapped to unselected rows
#' @param color_true The color that should be mapped to selected rows
#'
#' @examples
#' \dontrun{
#' sd <- SharedData$new(iris)
#' renderPlot({
#'   df <- sd$data(withSelection = TRUE, withFilter = TRUE)
#'   ggplot(df, aes(Sepal.Length, Sepal.Width,
#'     color = selection_factor(df))) +
#'     geom_point() +
#'     scale_color_selection("#444444", "skyblue1")
#' })
#'
#' }
#' @export
scale_fill_selection <- function(color_false, color_true) {
  list(
    ggplot2::scale_fill_manual(values = c("TRUE" = color_true, "FALSE" = color_false)),
    ggplot2::guides(fill = FALSE)
  )
}

#' @rdname scale_fill_selection
#' @export
scale_color_selection <- function(color_false, color_true) {
  list(
    ggplot2::scale_color_manual(values = c("TRUE" = color_true, "FALSE" = color_false)),
    ggplot2::guides(colour = FALSE)
  )
}

#' @param x Either a data frame with a \code{selected_} column, or, a logical
#'   vector indicating which rows are selected
#' @param na.replace The value to use to replace \code{NA} values; choose either
#'   \code{FALSE}, \code{NA}, or \code{TRUE} based on how you want values to be
#'   treated when no selection is active
#' @param reverse Whether the factor level order should be \code{c(FALSE, TRUE)}
#'   (normal) or \code{c(TRUE, FALSE)} (reverse). The former is required for
#'   ggplot2 2.2.0+, the latter for earlier versions.
#' @rdname scale_fill_selection
#' @export
selection_factor <- function(x, na.replace = c(FALSE, NA, TRUE),
  reverse = packageVersion("ggplot2") < "2.2.0") {

  if (missing(na.replace))
    na.replace <- FALSE

  selection <- if (is.logical(x)) {
    x
  } else {
    x$selected_
  }
  selection[is.na(selection)] <- na.replace

  lvls <- if (reverse) {
    c(TRUE, FALSE)
  } else {
    c(FALSE, TRUE)
  }

  factor(selection, ordered = TRUE, levels = lvls)
}

#' Synchronize Shiny brush selection with shared data
#'
#' Waits for a brush to change, and propagates that change to the
#' \code{sharedData} object.
#'
#' @param sharedData The shared data instance
#' @param brushId Character vector indicating the name of the \code{plotOutput}
#'   brush
#' @param ownerId (TBD)
#'
#' @export
maintain_selection <- function(sharedData, brushId, ownerId = "") {
  stopIfNotShiny("maintain_selection() requires the shiny package")

  force(sharedData)
  force(brushId)
  session <- shiny::getDefaultReactiveDomain()

  shiny::observeEvent(session$input[[brushId]], {
    df <- sharedData$data(withKey = TRUE, withFilter = TRUE)
    df <- shiny::brushedPoints(df, session$input[[brushId]])
    sharedData$selection(df$key_, ownerId)
  }, ignoreNULL = FALSE)
}

bootstrapLib <- function(theme = NULL) {
  # Intentionally use an older version of bootstrap. The rendering
  # environment may use a bootstrap version that has a theme, and
  # we don't want to trump that just for our little controls.
  # Ideally we should find a better solution for this.
  htmlDependency(
    name = "bootstrap",
    version = "3.3.2",
    package = "crosstalk",
    src = file.path("lib", "bootstrap"),
    script = "js/bootstrap.min.js",
    stylesheet = if (is.null(theme)) "css/bootstrap.min.css",
    meta = list(viewport = "width=device-width, initial-scale=1")
  )
}

selectizeLib <- function(bootstrap = TRUE) {
  htmlDependency(
    name = "selectize",
    version = "0.11.2",
    package = "crosstalk",
    src = "lib/selectize",
    stylesheet = if (bootstrap) "css/selectize.bootstrap3.css",
    script = "js/selectize.min.js"
  )
}

jqueryLib <- function() {
  htmlDependency(
    name = "jquery",
    version = "3.5.1",
    package = "crosstalk",
    src = "lib/jquery",
    script = "jquery.min.js"
  )
}

ionrangesliderLibs <- function() {
  list(
    jqueryLib(),
    htmlDependency(
      name = "ionrangeslider",
      version = "2.1.2",
      package = "crosstalk",
      src = "lib/ionrangeslider",
      script = "js/ion.rangeSlider.min.js",
      # ion.rangeSlider also needs normalize.css, which is already included in
      # Bootstrap.
      stylesheet = c("css/ion.rangeSlider.css",
        "css/ion.rangeSlider.skinShiny.css")
    ),
    htmlDependency(
      name = "strftime",
      version = "0.9.2",
      package = "crosstalk",
      src = "lib/strftime",
      script = "strftime-min.js"
    )
  )
}

makeGroupOptions <- function(sharedData, group, allLevels) {
  df <- sharedData$data(
    withSelection = FALSE,
    withFilter = FALSE,
    withKey = TRUE
  )

  if (inherits(group, "formula"))
    group <- lazyeval::f_eval(group, df)

  if (length(group) < 1) {
    stop("Can't form options with zero-length group vector")
  }

  lvls <- if (is.factor(group)) {
    if (allLevels) {
      levels(group)
    } else {
      levels(droplevels(group))
    }
  } else {
    sort(unique(group))
  }
  matches <- match(group, lvls)
  vals <- lapply(1:length(lvls), function(i) {
    df$key_[which(matches == i)]
  })

  lvls_str <- as.character(lvls)

  options <- list(
    items = data.frame(value = lvls_str, label = lvls_str, stringsAsFactors = FALSE),
    map = setNames(vals, lvls_str),
    group = sharedData$groupName()
  )

  options
}

#' Categorical filter controls
#'
#' Creates a select box or list of checkboxes, for filtering a
#' \code{\link{SharedData}} object based on categorical data.
#'
#' @param id An HTML element ID; must be unique within the web page
#' @param label A human-readable label
#' @param sharedData \code{SharedData} object with the data to filter
#' @param group A one-sided formula whose values will populate this select box.
#'   Generally this should be a character or factor column; if not, it will be
#'   coerced to character.
#' @param allLevels If the vector described by \code{group} is factor-based,
#'   should all the levels be displayed as options, or only ones that are
#'   present in the data?
#' @param multiple Can multiple values be selected?
#' @param columns Number of columns the options should be arranged into.
#'
#' @examples
#' ## Only run examples in interactive R sessions
#' if (interactive()) {
#'
#' sd <- SharedData$new(chickwts)
#' filter_select("feedtype", "Feed type", sd, "feed")
#'
#' }
#'
#' @export
filter_select <- function(id, label, sharedData, group, allLevels = FALSE,
  multiple = TRUE) {

  options <- makeGroupOptions(sharedData, group, allLevels)

  htmltools::browsable(attachDependencies(
    tags$div(id = id, class = "form-group crosstalk-input-select crosstalk-input",
      tags$label(class = "control-label", `for` = id, label),
      tags$div(
        tags$select(
          multiple = if (multiple) NA else NULL
        ),
        tags$script(type = "application/json",
          `data-for` = id,
          jsonlite::toJSON(options, dataframe = "columns", pretty = TRUE)
        )
      )
    ),
    c(list(jqueryLib(), bootstrapLib(), selectizeLib()), crosstalkLibs())
  ))
}

columnize <- function(columnCount, elements) {
  if (columnCount <= 1 || length(elements) <= 1) {
    return(elements)
  }

  columnSize <- ceiling(length(elements) / columnCount)
  lapply(1:ceiling(length(elements) / columnSize), function(i) {
    tags$div(class = "crosstalk-options-column",
      {
        start <- (i-1) * columnSize + 1
        end <- i * columnSize
        elements[start:end]
      }
    )
  })
}

#' @param inline If \code{TRUE}, render checkbox options horizontally instead of vertically.
#'
#' @rdname filter_select
#' @export
filter_checkbox <- function(id, label, sharedData, group, allLevels = FALSE, inline = FALSE, columns = 1) {
  options <- makeGroupOptions(sharedData, group, allLevels)

  labels <- options$items$label
  values <- options$items$value
  options$items <- NULL # Doesn't need to be serialized for this type of control

  makeCheckbox <- if (inline) inlineCheckbox else blockCheckbox

  htmltools::browsable(attachDependencies(
    tags$div(id = id, class = "form-group crosstalk-input-checkboxgroup crosstalk-input",
      tags$label(class = "control-label", `for` = id, label),
      tags$div(class = "crosstalk-options-group",
        columnize(columns,
          mapply(labels, values, FUN = function(label, value) {
            makeCheckbox(id, value, label)
          }, SIMPLIFY = FALSE, USE.NAMES = FALSE)
        )
      ),
      tags$script(type = "application/json",
        `data-for` = id,
        jsonlite::toJSON(options, dataframe = "columns", pretty = TRUE)
      )
    ),
    c(list(jqueryLib(), bootstrapLib()), crosstalkLibs())
  ))
}

blockCheckbox <- function(id, value, label) {
  tags$div(class = "checkbox",
    tags$label(
      tags$input(type = "checkbox", name = id, value = value),
      tags$span(label)
    )
  )
}

inlineCheckbox <- function(id, value, label) {
  tags$label(class = "checkbox-inline",
    tags$input(type = "checkbox", name = id, value = value),
    tags$span(label)
  )
}

#' Range filter control
#'
#' Creates a slider widget that lets users filter observations based on a range
#' of values.
#'
#' @param id An HTML element ID; must be unique within the web page
#' @param label A human-readable label
#' @param sharedData \code{SharedData} object with the data to filter
#' @param column A one-sided formula whose values will be used for this slider.
#'   The column must be of type \code{\link{Date}}, \code{\link{POSIXt}}, or
#'   numeric.
#' @param step Specifies the interval between each selectable value on the
#'   slider (if \code{NULL}, a heuristic is used to determine the step size). If
#'   the values are dates, \code{step} is in days; if the values are times
#'   (POSIXt), \code{step} is in seconds.
#' @param round \code{TRUE} to round all values to the nearest integer;
#'   \code{FALSE} if no rounding is desired; or an integer to round to that
#'   number of decimal places (for example, 1 will round to the nearest 0.1, and
#'   -2 will round to the nearest 100). Any rounding will be applied after
#'   snapping to the nearest step.
#' @param ticks \code{FALSE} to hide tick marks, \code{TRUE} to show them
#'   according to some simple heuristics.
#' @param animate \code{TRUE} to show simple animation controls with default
#'   settings; \code{FALSE} not to; or a custom settings list, such as those
#'   created using \code{\link[shiny]{animationOptions}}.
#' @param width The width of the slider control (see
#'   \code{\link[htmltools]{validateCssUnit}} for valid formats)
#' @param sep Separator between thousands places in numbers.
#' @param pre A prefix string to put in front of the value.
#' @param post A suffix string to put after the value.
#' @param timeFormat Only used if the values are Date or POSIXt objects. A time
#'   format string, to be passed to the Javascript strftime library. See
#'   \url{https://github.com/samsonjs/strftime} for more details. The allowed
#'   format specifications are very similar, but not identical, to those for R's
#'   \code{\link{strftime}} function. For Dates, the default is \code{"\%F"}
#'   (like \code{"2015-07-01"}), and for POSIXt, the default is \code{"\%F \%T"}
#'   (like \code{"2015-07-01 15:32:10"}).
#' @param timezone Only used if the values are POSIXt objects. A string
#'   specifying the time zone offset for the displayed times, in the format
#'   \code{"+HHMM"} or \code{"-HHMM"}. If \code{NULL} (the default), times will
#'   be displayed in the browser's time zone. The value \code{"+0000"} will
#'   result in UTC time.
#' @param dragRange This option is used only if it is a range slider (with two
#'   values). If \code{TRUE} (the default), the range can be dragged. In other
#'   words, the min and max can be dragged together. If \code{FALSE}, the range
#'   cannot be dragged.
#' @param min The leftmost value of the slider. By default, set to the minimal
#'   number in input data.
#' @param max The rightmost value of the slider. By default, set to the maximal
#'   number in input data.
#' @examples
#' ## Only run examples in interactive R sessions
#' if (interactive()) {
#'
#' sd <- SharedData$new(mtcars)
#' filter_slider("mpg", "Miles per gallon", sd, "mpg")
#'
#' }
#' @export
filter_slider <- function(id, label, sharedData, column, step = NULL,
  round = FALSE, ticks = TRUE, animate = FALSE, width = NULL, sep = ",",
  pre = NULL, post = NULL, timeFormat = NULL,
  timezone = NULL, dragRange = TRUE, min = NULL, max = NULL)
{
  # TODO: Check that this works well with factors
  # TODO: Handle empty data frame, NA/NaN/Inf/-Inf values

  if (is.character(column)) {
    column <- lazyeval::f_new(as.symbol(column))
  }

  df <- sharedData$data(withKey = TRUE)
  col <- lazyeval::f_eval(column, df)
  values <- na.omit(col)
  if (is.null(min))
    min <- min(values)
  if (is.null(max))
    max <- max(values)
  value <- range(values)

  ord <- order(col)
  options <- list(
    values = col[ord],
    keys = df$key_[ord],
    group = sharedData$groupName()
  )

  # If step is NULL, use heuristic to set the step size.
  findStepSize <- function(min, max, step) {
    if (!is.null(step)) return(step)

    range <- max - min
    # If short range or decimals, use continuous decimal with ~100 points
    if (range < 2 || hasDecimals(min) || hasDecimals(max)) {
      step <- pretty(c(min, max), n = 100)
      step[2] - step[1]
    } else {
      1
    }
  }

  if (inherits(min, "Date")) {
    if (!inherits(max, "Date") || !inherits(value, "Date"))
      stop("`min`, `max`, and `value must all be Date or non-Date objects")
    dataType <- "date"

    if (is.null(timeFormat))
      timeFormat <- "%F"

  } else if (inherits(min, "POSIXt")) {
    if (!inherits(max, "POSIXt") || !inherits(value, "POSIXt"))
      stop("`min`, `max`, and `value must all be POSIXt or non-POSIXt objects")
    dataType <- "datetime"

    if (is.null(timeFormat))
      timeFormat <- "%F %T"

  } else {
    dataType <- "number"
  }

  if (isTRUE(round))
    round <- 0
  else if (!is.numeric(round))
    round <- NULL
  step <- findStepSize(min, max, step)
  # Avoid ugliness from floating point errors, e.g.
  # findStepSize(min(quakes$mag), max(quakes$mag), NULL)
  # was returning 0.01999999999999957 instead of 0.2
  step <- signif(step, 14)

  if (dataType %in% c("date", "datetime")) {
    # For Dates, this conversion uses midnight on that date in UTC
    to_ms <- function(x) 1000 * as.numeric(as.POSIXct(x))

    # Convert values to milliseconds since epoch (this is the value JS uses)
    # Find step size in ms
    step  <- to_ms(max) - to_ms(max - step)
    min   <- to_ms(min)
    max   <- to_ms(max)
    value <- to_ms(value)
  }

  range <- max - min

  # Try to get a sane number of tick marks
  if (ticks) {
    n_steps <- range / step

    # Make sure there are <= 10 steps.
    # n_ticks can be a noninteger, which is good when the range is not an
    # integer multiple of the step size, e.g., min=1, max=10, step=4
    scale_factor <- ceiling(n_steps / 10)
    n_ticks <- n_steps / scale_factor

  } else {
    n_ticks <- NULL
  }

  sliderProps <- dropNulls(list(
    `data-type` = if (length(value) > 1) "double",
    `data-min` = formatNoSci(min),
    `data-max` = formatNoSci(max),
    `data-from` = formatNoSci(value[1]),
    `data-to` = if (length(value) > 1) formatNoSci(value[2]),
    `data-step` = formatNoSci(step),
    `data-grid` = ticks,
    `data-grid-num` = n_ticks,
    `data-grid-snap` = FALSE,
    `data-prettify-separator` = sep,
    `data-prefix` = pre,
    `data-postfix` = post,
    `data-keyboard` = TRUE,
    `data-keyboard-step` = step / (max - min) * 100,
    `data-drag-interval` = dragRange,
    `data-round` = round,
    # The following are ignored by the ion.rangeSlider, but are used by Shiny.
    `data-data-type` = dataType,
    `data-time-format` = timeFormat,
    `data-timezone` = timezone
  ))

  # Replace any TRUE and FALSE with "true" and "false"
  sliderProps <- lapply(sliderProps, function(x) {
    if (identical(x, TRUE)) "true"
    else if (identical(x, FALSE)) "false"
    else x
  })

  sliderTag <- div(
    class = "form-group crosstalk-input",
    class = "crosstalk-input-slider js-range-slider",
    id = id,

    style = if (!is.null(width)) paste0("width: ", validateCssUnit(width), ";"),
    if (!is.null(label)) controlLabel(id, label),
    do.call(tags$input, sliderProps),
    tags$script(type = "application/json",
      `data-for` = id,
      jsonlite::toJSON(options, dataframe = "columns", pretty = TRUE)
    )
  )

  # Add animation buttons
  if (identical(animate, TRUE))
    animate <- shiny::animationOptions()

  if (!is.null(animate) && !identical(animate, FALSE)) {
    if (is.null(animate$playButton))
      animate$playButton <- shiny::icon('play', lib = 'glyphicon')
    if (is.null(animate$pauseButton))
      animate$pauseButton <- shiny::icon('pause', lib = 'glyphicon')

    sliderTag <- tagAppendChild(
      sliderTag,
      tags$div(class='slider-animate-container',
        tags$a(href='#',
          class='slider-animate-button',
          'data-target-id'=id,
          'data-interval'=animate$interval,
          'data-loop'=animate$loop,
          span(class = 'play', animate$playButton),
          span(class = 'pause', animate$pauseButton)
        )
      )
    )
  }

  htmltools::browsable(attachDependencies(
    sliderTag,
    c(ionrangesliderLibs(), crosstalkLibs())
  ))
}

hasDecimals <- function(value) {
  truncatedValue <- round(value)
  return (!identical(value, truncatedValue))
}

#' @rdname filter_slider
#'
#' @param interval The interval, in milliseconds, between each animation step.
#' @param loop \code{TRUE} to automatically restart the animation when it
#'   reaches the end.
#' @param playButton Specifies the appearance of the play button. Valid values
#'   are a one-element character vector (for a simple text label), an HTML tag
#'   or list of tags (using \code{\link{tag}} and friends), or raw HTML (using
#'   \code{\link{HTML}}).
#' @param pauseButton Similar to \code{playButton}, but for the pause button.
#'
#' @export
animation_options <- function(interval=1000,
  loop=FALSE,
  playButton=NULL,
  pauseButton=NULL) {
  list(interval=interval,
    loop=loop,
    playButton=playButton,
    pauseButton=pauseButton)
}

#' Arrange HTML elements or widgets in Bootstrap columns
#'
#' This helper function makes it easy to put HTML elements side by side. It can
#' be called directly from the console but is especially designed to work in an
#' R Markdown document. Warning: This will bring in all of Bootstrap!
#'
#' @param ... \code{htmltools} tag objects, lists, text, HTML widgets, or
#'   NULL. These arguments should be unnamed.
#' @param widths The number of columns that should be assigned to each of the
#'   \code{...} elements (the total number of columns available is always 12).
#'   The width vector will be recycled if there are more \code{...} arguments.
#'   \code{NA} columns will evenly split the remaining columns that are left
#'   after the widths are recycled and non-\code{NA} values are subtracted.
#' @param device The class of device which is targeted by these widths; with
#'   smaller screen sizes the layout will collapse to a one-column,
#'   top-to-bottom display instead. xs: never collapse, sm: collapse below
#'   768px, md: 992px, lg: 1200px.
#'
#' @return A \code{\link[htmltools]{browsable}} HTML element.
#'
#' @examples
#' \donttest{
#' library(htmltools)
#'
#' # If width is unspecified, equal widths will be used
#' bscols(
#'   div(style = css(width="100%", height="400px", background_color="red")),
#'   div(style = css(width="100%", height="400px", background_color="blue"))
#' )
#'
#' # Use NA to absorb remaining width
#' bscols(widths = c(2, NA, NA),
#'   div(style = css(width="100%", height="400px", background_color="red")),
#'   div(style = css(width="100%", height="400px", background_color="blue")),
#'   div(style = css(width="100%", height="400px", background_color="green"))
#' )
#'
#' # Recycling widths
#' bscols(widths = c(2, 4),
#'   div(style = css(width="100%", height="400px", background_color="red")),
#'   div(style = css(width="100%", height="400px", background_color="blue")),
#'   div(style = css(width="100%", height="400px", background_color="red")),
#'   div(style = css(width="100%", height="400px", background_color="blue"))
#' )
#' }
#' @export
bscols <- function(..., widths = NA, device = c("xs", "sm", "md", "lg")) {
  device <- match.arg(device)

  if (length(list(...)) == 0) {
    widths = c()
  } else {
    if (length(widths) > length(list(...))) {
      warning("Too many widths provided to bscols; truncating")
    }
    widths <- rep_len(widths, length(list(...)))

    if (any(is.na(widths))) {
      remaining <- 12 - sum(widths, na.rm = TRUE)
      stretch_cols <- length(which(is.na(widths)))
      stretch_width <- max(1, floor(remaining / stretch_cols))
      widths[is.na(widths)] <- stretch_width
    }

    if (sum(widths) > 12) {
      warning("Sum of bscol width units is greater than 12")
    }
  }

  ui <- tags$div(class = "container-fluid crosstalk-bscols",
    # Counteract knitr pre/code output blocks
    tags$div(class = "fluid-row",
      unname(mapply(list(...), widths, FUN = function(el, width) {
        div(class = sprintf("col-%s-%s", device, width),
          el
        )
      }, SIMPLIFY = FALSE))
    )
  )

  browsable(attachDependencies(ui, list(jqueryLib(), bootstrapLib())))
}

controlLabel <- function(controlName, label) {
  if (is.null(label)) {
    NULL
  } else {
    tags$label(class = "control-label", `for` = controlName, label)
  }
}

# Given a vector or list, drop all the NULL items in it
dropNulls <- function(x) {
  x[!vapply(x, is.null, FUN.VALUE=logical(1))]
}

# Format a number without sci notation, and keep as many digits as possible (do
# we really need to go beyond 15 digits?)
formatNoSci <- function(x) {
  if (is.null(x)) return(NULL)
  format(x, scientific = FALSE, digits = 15)
}

test_that("crosstalk.min.css has been built", {

  src <- system.file("www/scss/crosstalk.scss", package = "crosstalk")

  new_css <- sass::sass(
    sass::sass_file(src),
    cache = NULL,
    options = sass::sass_options(output_style = "compressed")
  )

  # Remove class and attributes
  new_css <- sub("\n", "", as.character(new_css), fixed = TRUE)

  pkg_css_file <- system.file("www/css/crosstalk.min.css", package = "crosstalk")
  pkg_css <- readLines(pkg_css_file)

  # If this fails, that means that tools/updateCrosstalkCss.R needs to be run.
  expect_identical(new_css, pkg_css)
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shiny-shims.R
\name{getDefaultReactiveDomain}
\alias{getDefaultReactiveDomain}
\title{Get default reactive domain}
\usage{
getDefaultReactiveDomain()
}
\description{
Pass-through to [shiny::getDefaultReactiveDomain()], unless the shiny package
is not installed, in which case `NULL` is returned.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/crosstalk.R
\name{ClientValue}
\alias{ClientValue}
\title{ClientValue object}
\description{
An object that can be used in a \href{https://shiny.rstudio.com/}{Shiny} server
function to get or set a crosstalk variable that exists on the client. The
client copy of the variable is the canonical copy, so there is no direct
"set" method that immediately changes the value; instead, there is a
`sendUpdate` method that sends a request to the browser to change the value,
which will then cause the new value to be relayed back to the server.

This object is used to implement \code{\link{SharedData}} and should not need
to be used directly by users.
}
\examples{
library(shiny)

server <- function(input, output, session) {
  cv <- ClientValue$new("var1", "group1")

  r <- reactive({
    # Don't proceed unless cv$get() is a non-NULL value
    validate(need(cv$get(), message = FALSE))

    runif(cv$get())
  })

  observeEvent(input$click, {
    cv$sendUpdate(NULL)
  })
}

}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-new}{\code{ClientValue$new()}}
\item \href{#method-get}{\code{ClientValue$get()}}
\item \href{#method-sendUpdate}{\code{ClientValue$sendUpdate()}}
\item \href{#method-clone}{\code{ClientValue$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-new"></a>}}
\if{latex}{\out{\hypertarget{method-new}{}}}
\subsection{Method \code{new()}}{
Creates a new ClientValue object to reflect the crosstalk variable
specified by `group` and `name`.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ClientValue$new(
  name,
  group = "default",
  session = shiny::getDefaultReactiveDomain()
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{name}}{The name of the crosstalk variable.}

\item{\code{group}}{The name of the crosstalk variable group.}

\item{\code{session}}{The Shiny session to connect to; defaults to the current
session.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-get"></a>}}
\if{latex}{\out{\hypertarget{method-get}{}}}
\subsection{Method \code{get()}}{
Read the value. This is a reactive operation akin to reading a reactive
value, and so can only be done in a reactive context (e.g. in a
`shiny::reactive()`, `shiny::observe()`, or `shiny::isolate()` block).
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ClientValue$get()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-sendUpdate"></a>}}
\if{latex}{\out{\hypertarget{method-sendUpdate}{}}}
\subsection{Method \code{sendUpdate()}}{
Send a message to the browser asking it to update the crosstalk var to
the given value. This update does not happen synchronously, that is, a
call to `get()` immediately following `sendUpdate(value)` will not
reflect the new value.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ClientValue$sendUpdate(value)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{value}}{The new value for the crosstalk variable. Must be
serializable as JSON using `jsonlite`.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-clone"></a>}}
\if{latex}{\out{\hypertarget{method-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ClientValue$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggplot2.R
\name{maintain_selection}
\alias{maintain_selection}
\title{Synchronize Shiny brush selection with shared data}
\usage{
maintain_selection(sharedData, brushId, ownerId = "")
}
\arguments{
\item{sharedData}{The shared data instance}

\item{brushId}{Character vector indicating the name of the \code{plotOutput}
brush}

\item{ownerId}{(TBD)}
}
\description{
Waits for a brush to change, and propagates that change to the
\code{sharedData} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/controls.R
\name{filter_slider}
\alias{filter_slider}
\alias{animation_options}
\title{Range filter control}
\usage{
filter_slider(
  id,
  label,
  sharedData,
  column,
  step = NULL,
  round = FALSE,
  ticks = TRUE,
  animate = FALSE,
  width = NULL,
  sep = ",",
  pre = NULL,
  post = NULL,
  timeFormat = NULL,
  timezone = NULL,
  dragRange = TRUE,
  min = NULL,
  max = NULL
)

animation_options(
  interval = 1000,
  loop = FALSE,
  playButton = NULL,
  pauseButton = NULL
)
}
\arguments{
\item{id}{An HTML element ID; must be unique within the web page}

\item{label}{A human-readable label}

\item{sharedData}{\code{SharedData} object with the data to filter}

\item{column}{A one-sided formula whose values will be used for this slider.
The column must be of type \code{\link{Date}}, \code{\link{POSIXt}}, or
numeric.}

\item{step}{Specifies the interval between each selectable value on the
slider (if \code{NULL}, a heuristic is used to determine the step size). If
the values are dates, \code{step} is in days; if the values are times
(POSIXt), \code{step} is in seconds.}

\item{round}{\code{TRUE} to round all values to the nearest integer;
\code{FALSE} if no rounding is desired; or an integer to round to that
number of decimal places (for example, 1 will round to the nearest 0.1, and
-2 will round to the nearest 100). Any rounding will be applied after
snapping to the nearest step.}

\item{ticks}{\code{FALSE} to hide tick marks, \code{TRUE} to show them
according to some simple heuristics.}

\item{animate}{\code{TRUE} to show simple animation controls with default
settings; \code{FALSE} not to; or a custom settings list, such as those
created using \code{\link[shiny]{animationOptions}}.}

\item{width}{The width of the slider control (see
\code{\link[htmltools]{validateCssUnit}} for valid formats)}

\item{sep}{Separator between thousands places in numbers.}

\item{pre}{A prefix string to put in front of the value.}

\item{post}{A suffix string to put after the value.}

\item{timeFormat}{Only used if the values are Date or POSIXt objects. A time
format string, to be passed to the Javascript strftime library. See
\url{https://github.com/samsonjs/strftime} for more details. The allowed
format specifications are very similar, but not identical, to those for R's
\code{\link{strftime}} function. For Dates, the default is \code{"\%F"}
(like \code{"2015-07-01"}), and for POSIXt, the default is \code{"\%F \%T"}
(like \code{"2015-07-01 15:32:10"}).}

\item{timezone}{Only used if the values are POSIXt objects. A string
specifying the time zone offset for the displayed times, in the format
\code{"+HHMM"} or \code{"-HHMM"}. If \code{NULL} (the default), times will
be displayed in the browser's time zone. The value \code{"+0000"} will
result in UTC time.}

\item{dragRange}{This option is used only if it is a range slider (with two
values). If \code{TRUE} (the default), the range can be dragged. In other
words, the min and max can be dragged together. If \code{FALSE}, the range
cannot be dragged.}

\item{min}{The leftmost value of the slider. By default, set to the minimal
number in input data.}

\item{max}{The rightmost value of the slider. By default, set to the maximal
number in input data.}

\item{interval}{The interval, in milliseconds, between each animation step.}

\item{loop}{\code{TRUE} to automatically restart the animation when it
reaches the end.}

\item{playButton}{Specifies the appearance of the play button. Valid values
are a one-element character vector (for a simple text label), an HTML tag
or list of tags (using \code{\link{tag}} and friends), or raw HTML (using
\code{\link{HTML}}).}

\item{pauseButton}{Similar to \code{playButton}, but for the pause button.}
}
\description{
Creates a slider widget that lets users filter observations based on a range
of values.
}
\examples{
## Only run examples in interactive R sessions
if (interactive()) {

sd <- SharedData$new(mtcars)
filter_slider("mpg", "Miles per gallon", sd, "mpg")

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/crosstalk.R
\name{crosstalkLibs}
\alias{crosstalkLibs}
\title{Crosstalk dependencies}
\usage{
crosstalkLibs()
}
\description{
List of \code{\link[htmltools]{htmlDependency}} objects necessary for
Crosstalk to function. Intended for widget authors.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/controls.R
\name{bscols}
\alias{bscols}
\title{Arrange HTML elements or widgets in Bootstrap columns}
\usage{
bscols(..., widths = NA, device = c("xs", "sm", "md", "lg"))
}
\arguments{
\item{...}{\code{htmltools} tag objects, lists, text, HTML widgets, or
NULL. These arguments should be unnamed.}

\item{widths}{The number of columns that should be assigned to each of the
\code{...} elements (the total number of columns available is always 12).
The width vector will be recycled if there are more \code{...} arguments.
\code{NA} columns will evenly split the remaining columns that are left
after the widths are recycled and non-\code{NA} values are subtracted.}

\item{device}{The class of device which is targeted by these widths; with
smaller screen sizes the layout will collapse to a one-column,
top-to-bottom display instead. xs: never collapse, sm: collapse below
768px, md: 992px, lg: 1200px.}
}
\value{
A \code{\link[htmltools]{browsable}} HTML element.
}
\description{
This helper function makes it easy to put HTML elements side by side. It can
be called directly from the console but is especially designed to work in an
R Markdown document. Warning: This will bring in all of Bootstrap!
}
\examples{
\donttest{
library(htmltools)

# If width is unspecified, equal widths will be used
bscols(
  div(style = css(width="100\%", height="400px", background_color="red")),
  div(style = css(width="100\%", height="400px", background_color="blue"))
)

# Use NA to absorb remaining width
bscols(widths = c(2, NA, NA),
  div(style = css(width="100\%", height="400px", background_color="red")),
  div(style = css(width="100\%", height="400px", background_color="blue")),
  div(style = css(width="100\%", height="400px", background_color="green"))
)

# Recycling widths
bscols(widths = c(2, 4),
  div(style = css(width="100\%", height="400px", background_color="red")),
  div(style = css(width="100\%", height="400px", background_color="blue")),
  div(style = css(width="100\%", height="400px", background_color="red")),
  div(style = css(width="100\%", height="400px", background_color="blue"))
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/controls.R
\name{filter_select}
\alias{filter_select}
\alias{filter_checkbox}
\title{Categorical filter controls}
\usage{
filter_select(id, label, sharedData, group, allLevels = FALSE, multiple = TRUE)

filter_checkbox(
  id,
  label,
  sharedData,
  group,
  allLevels = FALSE,
  inline = FALSE,
  columns = 1
)
}
\arguments{
\item{id}{An HTML element ID; must be unique within the web page}

\item{label}{A human-readable label}

\item{sharedData}{\code{SharedData} object with the data to filter}

\item{group}{A one-sided formula whose values will populate this select box.
Generally this should be a character or factor column; if not, it will be
coerced to character.}

\item{allLevels}{If the vector described by \code{group} is factor-based,
should all the levels be displayed as options, or only ones that are
present in the data?}

\item{multiple}{Can multiple values be selected?}

\item{inline}{If \code{TRUE}, render checkbox options horizontally instead of vertically.}

\item{columns}{Number of columns the options should be arranged into.}
}
\description{
Creates a select box or list of checkboxes, for filtering a
\code{\link{SharedData}} object based on categorical data.
}
\examples{
## Only run examples in interactive R sessions
if (interactive()) {

sd <- SharedData$new(chickwts)
filter_select("feedtype", "Feed type", sd, "feed")

}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/crosstalk.R
\name{is.SharedData}
\alias{is.SharedData}
\title{Check if an object is \code{SharedData}}
\usage{
is.SharedData(x)
}
\arguments{
\item{x}{The object that may or may not be an instance of \code{SharedData}}
}
\value{
logical
}
\description{
Check if an object is an instance of \code{\link{SharedData}} or not.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggplot2.R
\name{scale_fill_selection}
\alias{scale_fill_selection}
\alias{scale_color_selection}
\alias{selection_factor}
\title{ggplot2 helpers}
\usage{
scale_fill_selection(color_false, color_true)

scale_color_selection(color_false, color_true)

selection_factor(
  x,
  na.replace = c(FALSE, NA, TRUE),
  reverse = packageVersion("ggplot2") < "2.2.0"
)
}
\arguments{
\item{color_false}{The color that should be mapped to unselected rows}

\item{color_true}{The color that should be mapped to selected rows}

\item{x}{Either a data frame with a \code{selected_} column, or, a logical
vector indicating which rows are selected}

\item{na.replace}{The value to use to replace \code{NA} values; choose either
\code{FALSE}, \code{NA}, or \code{TRUE} based on how you want values to be
treated when no selection is active}

\item{reverse}{Whether the factor level order should be \code{c(FALSE, TRUE)}
(normal) or \code{c(TRUE, FALSE)} (reverse). The former is required for
ggplot2 2.2.0+, the latter for earlier versions.}
}
\description{
Add \code{scale_fill_selection()} or \code{scale_color_selection} to a ggplot
to customize the scale for fill or color, respectively, for linked brushing.
Use \code{selection_factor} to turn logical vectors representing selection,
to a factor with the levels ordered for use with ggplot2 bar stacking.
}
\examples{
\dontrun{
sd <- SharedData$new(iris)
renderPlot({
  df <- sd$data(withSelection = TRUE, withFilter = TRUE)
  ggplot(df, aes(Sepal.Length, Sepal.Width,
    color = selection_factor(df))) +
    geom_point() +
    scale_color_selection("#444444", "skyblue1")
})

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/crosstalk.R
\name{SharedData}
\alias{SharedData}
\title{Shared data frame}
\description{
An R6 class that represents a shared data frame, or sufficiently data
frame-like object.

The primary use for \code{SharedData} is to be passed to Crosstalk-compatible
widgets in place of a data frame. Each \code{SharedData$new(...)} call makes
a new "group" of widgets that link to each other, but not to widgets in other
groups. You can also use a \code{SharedData} object from Shiny code in order
to react to filtering and brushing from non-widget visualizations (like
ggplot2 plots).
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-new}{\code{SharedData$new()}}
\item \href{#method-origData}{\code{SharedData$origData()}}
\item \href{#method-groupName}{\code{SharedData$groupName()}}
\item \href{#method-key}{\code{SharedData$key()}}
\item \href{#method-data}{\code{SharedData$data()}}
\item \href{#method-selection}{\code{SharedData$selection()}}
\item \href{#method-clearSelection}{\code{SharedData$clearSelection()}}
\item \href{#method-clone}{\code{SharedData$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-new"></a>}}
\if{latex}{\out{\hypertarget{method-new}{}}}
\subsection{Method \code{new()}}{
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{SharedData$new(
  data,
  key = NULL,
  group = createUniqueId(4, prefix = "SharedData")
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{data}}{A data frame-like object, or a Shiny
\link[shiny:reactive]{reactive expression} that returns a data
frame-like object.}

\item{\code{key}}{Character vector or one-sided formula that indicates the name
  of the column that represents the key or ID of the data frame. These
  \emph{must} be unique, and ideally will be something intrinsic to the
  data (a proper ID) rather than a transient property like row index.

  If \code{NULL}, then \code{row.names(data)} will be used.}

\item{\code{group}}{The "identity" of the Crosstalk group that widgets will join
when you pass them this \code{SharedData} object. In some cases, you
will want to have multiple independent \code{SharedData} objects link
up to form a single web of widgets that all share selection and
filtering state; in those cases, you'll give those \code{SharedData}
objects the same group name. (One example: in Shiny, ui.R and server.R
might each need their own \code{SharedData} instance, even though
they're intended to represent a single group.)}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-origData"></a>}}
\if{latex}{\out{\hypertarget{method-origData}{}}}
\subsection{Method \code{origData()}}{
Return the data frame that was used to create this
\code{SharedData} instance. If a reactive expression, evaluate the
reactive expression. Equivalent to \code{SharedData$data(FALSE, FALSE,
FALSE)}.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{SharedData$origData()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-groupName"></a>}}
\if{latex}{\out{\hypertarget{method-groupName}{}}}
\subsection{Method \code{groupName()}}{
Returns the value of \code{group} that was used to create
this instance.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{SharedData$groupName()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-key"></a>}}
\if{latex}{\out{\hypertarget{method-key}{}}}
\subsection{Method \code{key()}}{
Returns the vector of key values. Filtering is not applied.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{SharedData$key()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-data"></a>}}
\if{latex}{\out{\hypertarget{method-data}{}}}
\subsection{Method \code{data()}}{
Return the data (or read and return the data if the data is a Shiny
reactive expression).

When running in Shiny, calling \code{data()} is a reactive operation that
will invalidate if the selection or filter change (assuming that
information was requested), or if the original data is a reactive
expression that has invalidated.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{SharedData$data(withSelection = FALSE, withFilter = TRUE, withKey = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{withSelection}}{If `TRUE`, add a \code{selection_} column with
logical values indicating which rows are in the current selection, or
\code{NA} if no selection is currently active.}

\item{\code{withFilter}}{If `TRUE` (the default), only return rows that are part
of the current filter settings, if any.}

\item{\code{withKey}}{If `TRUE`, add a \code{key_} column with the key values of
each row (normally not needed since the key is either one of the other
columns or else just the row names).}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-selection"></a>}}
\if{latex}{\out{\hypertarget{method-selection}{}}}
\subsection{Method \code{selection()}}{
Get or set the current selection in the client.

If called without arguments, returns a logical vector of rows that are
currently selected (brushed), or \code{NULL} if no selection exists.
Intended to be called from a Shiny reactive context, and invalidates
whenever the selection changes.

If called with one or two arguments, sets the selection based on the
given value indirectly, by sending the value to the web browser (assumes
an active Shiny app or Shiny R Markdown document).
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{SharedData$selection(value, ownerId = "")}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{value}}{If provided, a logical vector of `nrow(origData())` length,
indicating which rows are currently selected (brushed).}

\item{\code{ownerId}}{Set this argument to the `outputId` of a widget if
conceptually that widget "initiated" the selection (prevents that
widget from clearing its visual selection box, which is normally
cleared when the selection changes). For example, if setting the
selection based on a [shiny::plotOutput()] brush, then
`ownerId` should be the `outputId` of that `plotOutput`.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-clearSelection"></a>}}
\if{latex}{\out{\hypertarget{method-clearSelection}{}}}
\subsection{Method \code{clearSelection()}}{
Clears the selection indirectly, by sending an instruction
to the client that it should do so.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{SharedData$clearSelection(ownerId = "")}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{ownerId}}{See the [SharedData$selection()] method.}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-clone"></a>}}
\if{latex}{\out{\hypertarget{method-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{SharedData$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}
